﻿//DANI CSharp API
//Written by Jsoon Kim
//2.18.2011
//Ver 0.1

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.IO.Ports;

namespace DANI_CSharp
{
    public class API
    {
        private SerialPort port;
        private String error;
        private Char[] responseBuffer;
        private Int32 responseOffset;
        private const Int32 RESPONSE_BUFFER_SIZE = 1024;

        //Constructor
        public API()
        {
            this.port = new SerialPort(
                "NONE",
                9600,
                Parity.None,
                8,
                StopBits.One);
            this.port.Handshake = Handshake.XOnXOff;
            this.port.Encoding = Encoding.ASCII;

            this.error = "";
            this.responseBuffer = new Char[RESPONSE_BUFFER_SIZE];
            this.responseOffset = 0;
        }

        //Constructor
        public API(
            String portName,
            Int32 baudRate,
            System.IO.Ports.Parity parity,
            Int32 dataBits,
            System.IO.Ports.StopBits stopBits,
            System.IO.Ports.Handshake handshake)
        {
            this.port = new SerialPort(
                portName,
                baudRate,
                parity,
                dataBits,
                stopBits);
            this.port.Handshake = handshake;
            this.port.Encoding = Encoding.ASCII;

            this.error = "";
            this.responseBuffer = new Char[RESPONSE_BUFFER_SIZE];
            this.responseOffset = 0;
        }

        //Set port name setting
        public Int32 setPortName(String portName)
        {
            if (this.port.IsOpen)
                return -1000;
            else
                this.port.PortName = portName;
            return 0;
        }

        //Get port name setting
        public String getPortName()
        {
            return this.port.PortName;
        }

        //Set baud rate setting
        public Int32 setBaudRate(Int32 baudRate)
        {
            if (this.port.IsOpen)
                return -1000;
            else
                this.port.BaudRate = baudRate;
            return 0;
        }

        //Get baud rate setting
        public Int32 getBaudRate()
        {
            return this.port.BaudRate;
        }

        //Set parity setting
        public Int32 setParity(System.IO.Ports.Parity parity)
        {
            if (this.port.IsOpen)
                return -1000;
            else
                this.port.Parity = parity;
            return 0;
        }

        //Get parity setting
        public System.IO.Ports.Parity getParity()
        {
            return this.port.Parity;
        }

        //Set data bits setting
        public Int32 setDataBits(Int32 dataBits)
        {
            if (this.port.IsOpen)
                return -1000;
            else
                this.port.DataBits = dataBits;
            return 0;
        }

        //Get data bits setting
        public Int32 getDataBits()
        {
            return this.port.DataBits;
        }
        
        //Set stop bits setting
        public Int32 setStopBits(System.IO.Ports.StopBits stopBits)
        {
            if (this.port.IsOpen)
                return -1000;
            else
                this.port.StopBits = stopBits;
            return 0;
        }

        //Get stop bits setting
        public System.IO.Ports.StopBits getStopBits()
        {
            return this.port.StopBits;
        }

        //Set handshake setting
        public Int32 setHandshake(System.IO.Ports.Handshake handshake)
        {
            if (this.port.IsOpen)
                return -1000;
            else
                this.port.Handshake = handshake;
            return 0;
        }

        //Get handshake setting
        public System.IO.Ports.Handshake getHandshake()
        {
            return this.port.Handshake;
        }

        //Get port open status
        public Boolean getPortOpen()
        {
            return this.port.IsOpen;
        }

        //Return string containing last special error
        public String getLastError()
        {
            return this.error;
        }

        //Connect to remote module
        public Int32 Connect()
        {
            if (this.port.IsOpen)
                return -1001;
            else if (this.port.PortName == "")
                return -1002;
            else
            {
                try
                {
                    this.port.Open();
                }
                catch (Exception ex)
                {
                    this.error = ex.Message;
                    return -4000;
                }
            }

            return 0;
        }

        //Disconnect from remote module
        public Int32 Disconnect()
        {
            if (!this.port.IsOpen)
                return -1003;
            else
            {
                try
                {
                    this.port.Close();
                }
                catch (Exception ex)
                {
                    this.error = ex.Message;
                    return -4000;
                }
            }

            return 0;
        }

        //Send weight read command to remote module.  Weight data can be checked using CheckResponse
        public Int32 SendWeightReadCommand()
        {
            if (!this.port.IsOpen)
                return -1004;
            else
            {
                this.responseOffset = 0;
                this.port.WriteLine("ww");
            }

            return 0;
        }

        //Send recalibrate (set current weight to zero) command to remote module
        public Int32 SendRecalibrateCommand()
        {
            if (!this.port.IsOpen)
                return -1004;
            else
            {
                this.responseOffset = 0;
                this.port.WriteLine("cc");
            }

            return 0;
        }

        //Check the response from the remote module
        public Int32 CheckResponse(ref String response)
        {
            //Read any bytes in serial buffer
            if (!this.port.IsOpen)
                return -1005;
            else if (this.port.BytesToRead > 0)
            {
                if (this.port.BytesToRead > RESPONSE_BUFFER_SIZE)
                    return -2001;
                else
                {
                    //Read bytes to response array
                    Array.Clear(this.responseBuffer, 0, RESPONSE_BUFFER_SIZE);
                    this.port.Read(this.responseBuffer, 0, this.port.BytesToRead);
                    response = new String(this.responseBuffer);
                    return 0;
                }
            }
            else
                return -2000;
        }

        //Convert error code returned from any of the API functions to a string message
        public String ErrorCodeToMessage(Int32 errorCode)
        {
            switch (errorCode)
            {
                case 0:
                    return "No error.";
                case -1000:
                    return "Port is open; Cannot write to parameters.";
                case -1001:
                    return "Port is already open.";
                case -1002:
                    return "Port name undefined.";
                case -1003:
                    return "Port is already closed.";
                case -1004:
                    return "Port is closed; Cannot send commands.";
                case -1005:
                    return "Port is closed.";
                case -2000:
                    return "No response yet.  If there is no response for a while, try resending command.";
                case -2001:
                    return "Response exceeded maximum response length.  Try resending command.";
                case -2002:
                    return "Response cannot be parsed; may be corrupt or only partially transmitted.  Try resending command.";
                case -2003:
                    return "Corruption detected in response.  Try resending command.";
                case -4000:
                    return "Check returned string in getLastError().";
                default:
                    return "Undefined Error Code";
            }

            return "";
        }

        //Returns direct handle to serial port for advanced programming
        public SerialPort GetSerialPort()
        {
            return this.port;
        }

        //DEBUG - Write line to serial buffer
        public Int32 DEBUG_SendLine(String command)
        {
            if (this.port.IsOpen)
                this.port.WriteLine(command);
            else
                return -1;

            return 0;
        }

        //DEBUG - Read line from serial buffer
        public String DEBUG_ReadLine()
        {
            if (this.port.IsOpen &&
                this.port.BytesToRead > 0 &&
                this.port.BytesToRead <= RESPONSE_BUFFER_SIZE)
            {
                Array.Clear(this.responseBuffer, 0, RESPONSE_BUFFER_SIZE);
                this.port.Read(this.responseBuffer, 0, this.port.BytesToRead);
                return new String(this.responseBuffer);
            }

            return "";
        }
    }
}